@@ -71,6 +71,144 @@ class InfoWidget(WidgetABC):
         ) = NEW_INFO_SIGNAL.OTHER.value
         self.innerframe_for_recog = frame_labels_innerframe_for_recog = None
         self.current_clicked_label = None
+        self.info_template_name = None
+        self.info_template_copy_name = "info_template"
+        self.info_template_list = None
+        self.info_template_var = StringVar()
+        self.info_templates_combobox = None
+        self.info_template_dir_path = None
+        self.info_template_dir_path_copy_name = "info_template_dir"
+        self.info_template_file_exts = [".txt"]
+        self.open_info_template_dir_str = _("Open Directory")
+
+    def update_info_templates_combobox(self):
+        if not self.info_templates_combobox:
+            return
+        self.info_templates_combobox[
+            "values"
+        ] = self.get_info_template_list() + [self.open_info_template_dir_str]
+
+        pass
+
+    def get_info_template_dir_path(self):
+        if self.info_template_dir_path is None:
+            self.set_info_template_dir_path(update_ui=False)
+            if (
+                self.info_template_dir_path is None
+                or self.info_template_dir_path == ()
+            ):
+                _path = str(Path.home())
+                self.set_info_template_dir_path(
+                    self.info_template_dir_path_copy_name, _path
+                )
+
+        if not os.path.exists(self.info_template_dir_path):
+            print(_("Path does not exist."))
+            return str(Path.home())
+        return self.info_template_dir_path
+
+    def set_info_template_dir_path(self, _dir=None, update_ui=False):
+        if _dir == None:
+            _dir_copy = self.get_copy(
+                key=self.info_template_dir_path_copy_name
+            )
+            if _dir_copy == None or _dir_copy == ():
+                _dir = str(Path.home())
+            else:
+                if not os.path.exists(_dir_copy):
+                    print(_("Path does not exist."))
+                    _dir = str(Path.home())
+                else:
+                    _dir = _dir_copy
+        else:
+            self.set_copy(self.info_template_dir_path_copy_name, _dir)
+        #         _dir = _dir[0] if isinstance(_dir,tuple) else _dir
+        self.info_template_dir_path = _dir
+        if update_ui:
+            self.update_info_templates_combobox()
+        pass
+
+    def list_info_templates_dir(self):
+        if self.get_info_template_dir_path() is None:
+            return None
+        listed_contents = os.listdir(self.get_info_template_dir_path())
+        listed_contents = [
+            f
+            for f in listed_contents
+            if os.path.isfile(
+                os.path.join(self.get_info_template_dir_path(), f)
+            )
+        ]
+        new_listed_contents = []
+        for f in listed_contents:
+            for e in self.info_template_file_exts:
+                if f.lower().endswith(e):
+                    new_listed_contents.append(f)
+                    break
+        listed_contents = None
+        return new_listed_contents
+
+    def get_info_template_list(self, _refresh=False):
+        if not self.info_template_list or _refresh:
+            self.info_template_list = self.list_info_templates_dir()
+        if not self.info_template_list:
+            return []
+        return self.info_template_list
+
+    def update_info_template_list(self):
+        self.info_template_list = None
+        self.get_info_template_list()
+
+    # def set_info_template_name(self, name=None):
+    #    if not name:
+    #        return
+    #    self.info_template_name = name
+
+    def get_info_template_name(self):
+        name = None
+        if self.info_template_name:
+            return self.info_template_name
+
+        if self.info_template_dir_path_copy_name in self.get_copies_keys():
+            name = self.get_copy(self.info_template_dir_path_copy_name)
+            if info_template_exists(name):
+                return name
+
+        names = self.get_info_template_list()
+        if len(names) < 1:
+            return None
+        name = names[0]
+        return name
+
+    def get_info_template_path_by_name(self, name):
+        _path = os.path.join(self.get_info_template_dir_path(), name)
+        return _path
+
+    def get_info_template_content(self, name=None):
+        if not name:
+            name = self.get_info_template_name()
+            self.mk_tmsg(name)
+        if not name:
+            return None
+        assert not name is None
+        template_path = self.get_info_template_path_by_name(name)
+        if not os.path.exists(template_path):
+            return None
+
+        self.mk_tmsg(f"template_path\t{template_path}")
+        content = None
+        with open(template_path, "r", encoding="utf-8") as f:
+            content = f.read()
+        return content
+
+    def set_info_template_name(self, name=None, update_ui=False):
+        print(205, name)
+        if not name:
+            return
+        self.set_copy(self.info_template_copy_name, name)
+        self.info_template_name = name
+        if update_ui:
+            self.set_info_text_content_use_template()
 
     def get_frame_label_margin(self):
         return self.frame_label_margin
@@ -349,6 +487,7 @@ class InfoWidget(WidgetABC):
             # self.del_picked_frames_for_recog()
             self.delete_button_place()
             self.set_action(ACTION.READ)
+            self.place_forget_info_templates_combobox()
             pass
 
     def set_action_to_pick(self):
@@ -361,6 +500,7 @@ class InfoWidget(WidgetABC):
         if not self.is_action_recog():
             self.del_picked_frames()
             self.set_action(ACTION.RECOG)
+            self.place_forget_info_templates_combobox()
 
     def get_action(self):
         return self.fw.get_action()
@@ -525,6 +665,7 @@ class InfoWidget(WidgetABC):
         if not isinstance(frames, list):
             frames = [frames]
         self.picked_frames = frames
+
         if update_label:
             self.update_picked_frame_labels()
 
@@ -662,6 +803,8 @@ class InfoWidget(WidgetABC):
         self.del_saved_frames()
         self.del_picked_frames()
         self.clear_info_widget_area_content()
+        self.place_info_templates_combobox()
+        self.set_info_text_content_use_template()
 
     def update_widgets_by_info_id(self, info_id=None):
         self.clear_picked_frame_labels()
@@ -794,10 +937,10 @@ class InfoWidget(WidgetABC):
                 shutil.move(src, get_new_backup_path())
         self.set_basic_infos(to_none=True)
         self.update_saved_info_combobox_values()
-        self.del_showed_info()
+        self.clear_showed_info()
         self.set_msg(
-            _("Information of %(basic_info)(%(info_id)) is deleted.")
-            % (basic_info, info_id)
+            _("Information of %(basic_info)s (%(info_id)s) is deleted.")
+            % ({"basic_info": basic_info, "info_id": info_id})
         )
         pass
 
@@ -814,6 +957,82 @@ class InfoWidget(WidgetABC):
         self.fw.play_button_command()
         pass
 
+    def get_info_template_values(self):
+        template_names = self.get_info_template_list()
+        # template_names = [ Path(n).stem for n in template_names ]
+        template_names += [self.open_info_template_dir_str]
+        return template_names
+
+    def get_info_templates_combobox_x(self):
+        return self.get_x()
+
+    def get_info_templates_combobox_y(self):
+        return self.get_save_button_y()
+
+    def get_info_templates_combobox_width(self):
+        return int((self.get_save_button_x() - self.get_x()) * 0.7)
+
+    def get_info_templates_combobox_height(self):
+        return self.info_templates_combobox.winfo_reqheight()
+
+    def place_info_templates_combobox(self):
+        if not self.info_templates_combobox:
+            return
+        self.info_templates_combobox.place(
+            x=self.get_info_templates_combobox_x(),
+            y=self.get_info_templates_combobox_y(),
+            width=self.get_info_templates_combobox_width(),
+            height=self.get_info_templates_combobox_height(),
+        )
+        pass
+
+    def place_forget_info_templates_combobox(self):
+        if not self.info_templates_combobox:
+            return
+        self.info_templates_combobox.place_forget()
+        pass
+
+    def open_dir(self, dist="~"):
+        _open = (
+            sys.platform == "darwin"
+            and "open"
+            or sys.platform == "win32"
+            and "explorer"
+            or "xdg-open"
+        )
+        subprocess.Popen([_open, dist])
+        pass
+
+    def info_template_exists(self, name):
+        _exist = os.path.exists(
+            os.path.join(self.get_info_template_dir_path(), name)
+        )
+        return _exist
+
+    def info_template_var_trace_w(self, *args):
+        info_template_name = self.info_template_var.get()
+        # self.mk_tmsg(f"info_template_name\t{info_template_name}")
+        if info_template_name == self.open_info_template_dir_str:
+            # self.open_dir(get_info_templates_path())
+            _path = filedialog.askdirectory(
+                parent=self.root,
+                title=_("Select a template directory."),
+                initialdir=str(Path.home()),
+                mustexist=True,
+            )
+            if _path is None:
+                return
+            self.set_copy(self.info_template_dir_path_copy_name, _path)
+            self.set_info_template_dir_path(_dir=_path, update_ui=True)
+            return
+
+        if not self.info_template_exists(info_template_name):
+            self.update_info_template_list()
+            return
+        self.set_info_template_name(name=info_template_name, update_ui=True)
+        # self.set_info_text_content_use_template()
+        pass
+
     def set_widgets(self):
         super().set_widgets()
         self.set_frame_widget()
@@ -854,6 +1073,23 @@ class InfoWidget(WidgetABC):
         simpletooltip.create(
             self.info_text, _("Write additional information here.")
         )
+
+        self.info_templates_combobox = ttk.Combobox(
+            self.root,
+            textvariable=self.info_template_var,
+            values=self.get_info_template_values(),
+            justify="center",
+        )
+        simpletooltip.create(
+            self.info_templates_combobox,
+            _(
+                "Select a template file or "
+                + "a directory contains template files."
+                + ' Template files ends with ".txt".'
+            ),
+        )
+        self.info_template_var.trace("w", self.info_template_var_trace_w)
+
         self.save_button = ttk.Button(
             self.root, text=_("Save"), command=self.save_button_command
         )
@@ -1054,6 +1290,10 @@ class InfoWidget(WidgetABC):
 
         if self.is_action_read():
             self.delete_button_place()
+        if not self.get_info_id():
+            self.place_info_templates_combobox()
+        else:
+            self.place_forget_info_templates_combobox()
 
     def get_picked_frames_len(self):
         return len(self.picked_frames)
@@ -1081,12 +1321,26 @@ class InfoWidget(WidgetABC):
         if not isinstance(frames, list):
             frames = [frames]
         self.picked_frames = frames + self.picked_frames
+
+        info_id = self.get_info_id()
+        print("info_id", info_id)
+        if (not info_id) and self.get_picked_frames_len() < 2:
+            self.place_info_templates_combobox()
+            self.set_info_text_content_use_template()
+
         if update_label:
             new_added_frames = []
             if self.is_action_read():
                 pass
             self.set_picked_frame_labels_image(self.picked_frames)
 
+    def set_info_text_content_use_template(self):
+        content = self.get_info_template_content()
+        if not content:
+            return
+        self.set_info_text_content(content)
+        pass
+
     def get_info_frame(self):
         frame = ttk.Frame(self.info_scrolledframe_innerframe)
         return frame
@@ -1097,11 +1351,19 @@ class InfoWidget(WidgetABC):
     def set_picked_frame_labels_image(self, frames=None):
         self.set_frame_labels_image(frames)
 
+    def get_saved_frame_labels(self):
+        if len(self.saved_frame_labels) < 1:
+            return None
+        return self.saved_frame_labels
+
     def del_saved_frame_by_id(
-        self, frame_id=None, ask=True, update_widgets=True
+        self, frame_id=None, index=0, ask=True, update_widgets=True
     ):
         if frame_id is None:
             return
+
+        self.show_widget_click(self.get_saved_frame_labels(), index)
+
         if ask:
             if not messagebox.askyesno(
                 _("Delete saved frame"),
@@ -1190,14 +1452,16 @@ class InfoWidget(WidgetABC):
             return
         if self.is_test():
             print("saved_frames_len", len(saved_frames))
+        index = 0
         for _id, f in saved_frames:
             if self.is_test():
                 print("`saved_frame.id`", _id)
             label = ttk.Label(self.pick_scrolledframe_innerframe)
             self.set_label_image(f, label)
+            i = index
             label.bind(
                 "<Button-1>",
-                lambda event, _id=_id: self.del_saved_frame_by_id(_id),
+                lambda event, _id=_id, i=i: self.del_saved_frame_by_id(_id, i),
             )
             label.pack(
                 side="left",
@@ -1206,6 +1470,7 @@ class InfoWidget(WidgetABC):
             )
             simpletooltip.create(label, _("Saved frame, Click to delete."))
             self.saved_frame_labels.append(label)
+            index += 1
         pass
 
     def show_labels_image_by_frames(
@@ -1296,6 +1561,9 @@ class InfoWidget(WidgetABC):
 
     def recog_picked_frame_by_index(self, index=None, show_click=True):
         assert index is not None
+        if self.is_info_ids_none():
+            self.set_msg(_("Saved dateset doesn't exist."))
+            return
         if index is None:
             if self.is_test():
                 print("`recog_picked_frame_by_index.index` is `None`.")
@@ -1318,14 +1586,29 @@ class InfoWidget(WidgetABC):
         if show_click:
             labels = self.get_picked_frame_labels_for_recog()
 
-            for l in labels:
-                l.config(borderwidth=0, background="")
+            self.show_widget_click(labels, index)
 
-            label = labels[index]
-            label.config(
-                borderwidth=self.get_recog_picked_frame_labels_borderwidth(),
-                background="blue",
-            )
+        pass
+
+    def show_widget_click(self, widget_list=None, index=0):
+        self.show_widget_click_by_default(widget_list, index)
+        pass
+
+    def show_widget_click_by_default(
+        self, widget_list=None, index=0, borderwidth=8, background="blue"
+    ):
+
+        if widget_list is None:
+            return
+
+        for w in widget_list:
+            w.config(borderwidth=0, background="")
+
+        w = widget_list[index]
+        w.config(
+            borderwidth=borderwidth,
+            background=background,
+        )
 
         pass
 
@@ -1418,7 +1701,12 @@ class InfoWidget(WidgetABC):
         self.add_picked_frames(frames)
         pass
 
+    def is_info_ids_none(self):
+        return self.fw.is_info_ids_none()
+
     def recog_frame_by_default(self):
+        # if self.is_info_ids_none():
+        #    return
         self.clear_picked_frame_labels()
         self.clear_saved_frame_labels()
         frames = self.get_picked_frames_from_frame()
